{% extends "base/base_layout.html" %}
 {% block sidebar_option %}
      sidebar-collapse
{% endblock %}
{% block content %}
<div class="content-wrapper">
  <div class="content-header">
  </div>
   <div class="container-fluid">
        <div class="row">
            <div class="col-lg-12">
            <div class="card">
              <div class="card-body">
                <h1>Donate to MobSF Project</h1>
                <p class="lead">
                    You're Awesome! Thank you for your support ♥
                </p>
                <div class="row">

                    <div class="col-lg-6">
                      
                        <div class="card">
                            <div class="card-header ui-sortable-handle" >
                              <h3 class="card-title">
                            Github Sponsors
                              </h3>
                            </div><!-- /.card-header -->
                            <div class="card-body">
                                <p style="text-align: center;">
                                    Donate to MobSF project through Github Sponsors. Github will match the first $5000 in donations.
                                </br>
                                </br>
                                <iframe src="https://github.com/sponsors/ajinabraham/button" title="Sponsor ajinabraham" height="35" width="116" style="border: 0;"></iframe>
                                </p>
                            </div><!-- /.card-body -->
                          </div>
                    </div>
                    <div class="col-lg-6">
                    <div class="card">
                        <div class="card-header ui-sortable-handle" >
                          <h3 class="card-title">
                          Paypal Donations
                          </h3>
                        </div><!-- /.card-header -->
                        <div class="card-body">
                            <p style="text-align: center;">
                         You can also donate to MobSF project through PayPal.
                         </br>
                         </br>
                           <a href="https://www.paypal.com/paypalme/ajinabraham" rel="noopener noreferrer"><input type="image" src="https://www.paypalobjects.com/en_US/i/btn/btn_donateCC_LG.gif" border="0" name="submit" title="PayPal - The safer, easier way to pay online!" alt="Donate with PayPal button" /></a>
                        </p>
                        </div><!-- /.card-body -->
                    </div>
                    </div>
                </div>
          </div>
        </div>
       </div>
     </div>
    </div>
</div>
{% endblock %}
