@charset "UTF-8";

body {
    font-family: 'Open Sans', sans-serif;
}

#icon{
    width: 180px;
    height: 180px;
}
.app{
    display: block;
    margin-left: auto;
    margin-right: auto;
}

.container {
	top: 50%;
	left: 50%;
}
.basic {
  width: 100%;
}

.header {
  padding: 100px;
  text-align: center;
  background: #3C3E4B;
  color: white;
  font-size: 30px;
}
.htext {
  color: white;
  font-size: 34pt;
  width: 100%;
}

.center{
  width: 50%;
  display: block;
  margin-left: auto;
  margin-right: auto;
}


table.no_border, th.no_border, td.no_border {
  border: none;
  border-collapse: collapse;
}

table { page-break-inside:auto }
thead { display: table-header-group }
tfoot { display: table-row-group }
tr { page-break-inside: avoid }

table, th, td {
  border: 1px solid black;
  border-collapse: collapse;
}

th,
td {
	padding: 15px;
}
th{
    text-align: left;
    font-size: 15pt;
}
td{
   height: 10px;
}
.wrapped-text {
    white-space: pre-wrap;
}

.high {
  color: #F05239;
}
.danger {
  color: red;
}
.warning {
  color: orange;
}
.info {
  color: #2196f3;
}
.success {
  color: green;
}


.td-high {
  background-color: red;
}
.td-warning {
  background-color: orange;
}
.td-info {
  background-color: #2196f3;
}
.td-success {
  background-color: green;
}

.td-secondary {
  background-color: rgb(30, 31, 30);
}

.tbl-white {
  color:#ffffff;
}

.grade {
  text-align: center;
  font-size: 100px;
  color: white;
  line-height:150px;
  display: inline-block;
  height:150px;
  width: 150px;
}

.secondary {
  color: #6c757d;
}

html {
  color: #393939;
  font-size: 11pt;
  font-weight: 300;
  line-height: 1.5; }
html body h1 {
  color: #565656;
  font-size: 28pt;
  margin: 1cm 2cm 0 2cm;
  width: 100%; }
html body h2, html body h3, html body h4 {
  font-family: 'Oswald', sans-serif;
  color: #4d4d4d;
  font-weight: 600; }
html body h2 {
  font-size: 28pt;}
html body h3 {
  font-weight: 300;
  font-size: 15pt;
  margin:0;
  padding:0; }
html body h4 {
  font-size: 13pt; }
h5 {
  font-family: 'Oswald', sans-serif;
  font-size: 11pt; 
  display: inline;
}
