{% load static %}
<!DOCTYPE html>
<html>
<head>
  <meta charset="utf-8">
  <meta http-equiv="X-UA-Compatible" content="IE=edge">
  <title>Forbidden</title>
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <meta name="description" content="Mobile Security Framework - MobSF">
  <meta name="author" content="Ajin Abraham">
  <link rel="icon" href="{% static "img/favicon.ico" %}">

  <!-- Tell the browser to be responsive to screen width -->
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <!-- Theme style -->
  <link rel="stylesheet" href="{% static "adminlte/dashboard/css/adminlte.min.css" %}">
  <!-- Google Font: Source Sans Pro -->
  <link href="https://fonts.googleapis.com/css?family=Source+Sans+Pro:300,400,400i,700" rel="stylesheet">
</head>
<body class="hold-transition sidebar-collapse layout-fixed layout-navbar-fixed">
<div class="wrapper">
 {% include "base/nav.html" %}
<div class="content-wrapper">
    <div class="content-header">
    </div>
     <div class="container-fluid">
          <div class="row">
              <div class="col-lg-12">
              <div class="card">
                <div class="card-body">
                  <h2>Forbidden</h2>
                    <p class="lead">You do not have required permissions to perform this action.</p>
            </div>
          </div>
         </div>
       </div>
      </div>
  </div>

  <!-- /.content-wrapper -->
  <footer class="main-footer">
    <strong>© {% now "Y" %} Mobile Security Framework - MobSF | <a href="https://ajinabraham.com">Ajin Abraham</a> | <a href="https://opensecurity.in">OpenSecurity</a>.</strong>
  </footer>
 
</div>
<!-- ./wrapper -->

</body>
</html>
