{% load static %}
<!DOCTYPE html>
<html>
<head>
  <meta charset="utf-8">
  <meta http-equiv="X-UA-Compatible" content="IE=edge">
  <title>{{title}}</title>
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <meta name="description" content="Mobile Security Framework - MobSF">
  <meta name="author" content="Ajin Abraham">
  <link rel="icon" href="{% static "img/favicon.ico" %}">

  <!-- Tell the browser to be responsive to screen width -->
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <!-- Font Awesome -->
  <link rel="stylesheet" href="{% static "adminlte/plugins/fontawesome-free/css/all.min.css" %}">
  <!-- Theme style -->
  <link rel="stylesheet" href="{% static "adminlte/dashboard/css/adminlte.min.css" %}">
  <!-- overlayScrollbars -->
  <link rel="stylesheet" href="{% static "adminlte/plugins/overlayScrollbars/css/OverlayScrollbars.min.css" %}">
  <!-- Ionicons -->
  <link rel="stylesheet" href="https://code.ionicframework.com/ionicons/2.0.1/css/ionicons.min.css">
  <!-- Google Font: Source Sans Pro -->
  <link href="https://fonts.googleapis.com/css?family=Source+Sans+Pro:300,400,400i,700" rel="stylesheet">
  <style>
    body {
      zoom: 0.8; 
    }
    .modal-backdrop {
      position: fixed;
      top: 0;
      left: 0;
      width: 100%;
      height: 100%;
      z-index: 1040; /* Ensure it is above most elements but below modal */
      background-color: rgba(0, 0, 0, 0.5); /* Default backdrop color */
    }

  </style>
  {% block extra_css %}
 
  {% endblock %}
</head>
<body class="hold-transition {% block sidebar_option %} {% endblock %} layout-fixed layout-navbar-fixed">
<div class="wrapper">
{% include "base/nav.html" %}    
 <!-- sidebar-->
  <!-- Main Sidebar Container -->
  <aside class="main-sidebar sidebar-dark-primary elevation-4">
    <!-- Brand Logo -->
    <a href="{% url 'home' %}" class="brand-link">
      <img src="{% static "img/mobsf_icon.png" %}" alt="MobSF" class="brand-image"
           style="opacity: .8">
      <span class="brand-text logo-lg"> | <strong> Mob</strong>SF</span>
    </a>
    {% block sidebar %}
  
    {% endblock %}
  </aside>
 <!-- /.sidebar -->
  <!-- Content Wrapper. Contains page content -->
  {% block content %}
  
  {% endblock %}

  <!-- /.content-wrapper -->
  <footer class="main-footer">
    <strong>© {% now "Y" %} Mobile Security Framework - MobSF | <a href="https://ajinabraham.com">Ajin Abraham</a> | <a href="https://opensecurity.in">OpenSecurity</a>.</strong>
    <div class="float-right d-none d-sm-inline-block">
      <b>Version</b> {{ version }}
    </div>
  </footer>

  <!-- Control Sidebar -->
  <aside class="control-sidebar control-sidebar-dark">
  </aside>
  <!-- /.control-sidebar -->
</div>
<!-- ./wrapper -->
<div class="modal fade scan-logs-mdl" tabindex="-1" role="dialog" aria-labelledby="scan-logs-mdl-label" aria-hidden="true">
  <div class="modal-dialog modal-xl">
    <div class="modal-content">
      {% block scan_logs %}
      {% endblock %}
    </div>
  </div>
</div>


<!-- jQuery -->
<script src="{% static "adminlte/plugins/jquery.min.js" %}"></script>
<!-- jQuery UI 1.11.4 -->
<script src="{% static "adminlte/plugins/jquery-ui.min.js" %}"></script>
<!-- Resolve conflict in jQuery UI tooltip with Bootstrap tooltip -->
<script>
  $.widget.bridge('uibutton', $.ui.button)
  // Replace datatables alert with console.warn
  window.alert = (function() {
    var nativeAlert = window.alert;
    return function(message) {
        window.alert = nativeAlert;
        message.indexOf("DataTables warning") === 0 ?
            console.warn(message) :
            nativeAlert(message);
    }
})();
</script>
<!-- Bootstrap 4 -->
<script src="{% static "adminlte/plugins/bootstrap/bootstrap.bundle.min.js" %}"></script>
<!-- overlayScrollbars -->
<script src="{% static "adminlte/plugins/overlayScrollbars/js/jquery.overlayScrollbars.min.js" %}"></script>
<!-- AdminLTE App -->
<script src="{% static "adminlte/dashboard/js/adminlte.min.js" %}"></script>
<script>
// navbar hack
if ($(".sidebar-collapse")[0])
document.getElementById("nbar").style.display = "none";
</script>
{% block extra_scripts %}

{% endblock %}

</body>
</html>
