{% extends "base/base_layout.html" %}
 {% block sidebar_option %}
      sidebar-collapse
{% endblock %}
{% block content %}
<div class="content-wrapper">
  <div class="content-header">
  </div>
   <div class="container-fluid">
        <div class="row">
            <div class="col-lg-12">
            <div class="card">
              <div class="card-body">
                <h2>Error</h2>
                  <p class="lead">{{ exp }}</p>
                  <p>{{ doc }}</p>
          </div>
        </div>
       </div>
     </div>
    </div>
</div>
{% endblock %}
