{% extends "base/base_layout.html" %}
 {% block sidebar_option %}
      sidebar-collapse
{% endblock %}
{% block content %}
<div class="content-wrapper">
  <div class="content-header">
  </div>
   <div class="container-fluid">
        <div class="row">
            <div class="col-lg-12">
            <div class="card">
              <div class="card-body">
                <h1>About Mobile Security Framework</h1>
              
                  <p class="lead">
                    Mobile Security Framework (MobSF) is a security research platform for mobile applications in Android, iOS and Windows Mobile. MobSF can be used for a variety of use cases such as mobile application security, penetration testing, malware analysis, and privacy analysis. The Static Analyzer supports popular mobile app binaries like APK, IPA, APPX and source code. Meanwhile, the Dynamic Analyzer supports both Android and iOS applications and offers a platform for interactive instrumented testing, runtime data and network traffic analysis. MobSF seamlessly integrates with your DevSecOps or CI/CD pipeline, facilitated by REST APIs and CLI tools, enhancing your security workflow with ease.
                 </p>
                  <p> <strong>Author:</strong> <a href="//twitter.com/ajinabraham">Ajin Abraham</a> </p>      
                  <h2>Active Collaborators</h2>
                    <p><a href="//github.com/magaofei">Magaofei</a></p>
                    <p><a href="//github.com/matandobr">Matan Dobrushin</a></p>
                    <p><a href="//github.com/superpoussin22">Vincent Nadal</a></p>
          </div>
        </div>
       </div>
     </div>
    </div>
</div>
{% endblock %}
